# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"rename": "cli.ipynb",
         "fix_colab_links": "cli.ipynb",
         "REGEX": "cli.ipynb",
         "fix_nbs_colab_links": "cli.ipynb",
         "dbn_writer": "dbn.ipynb",
         "has_positions": "dbn.ipynb",
         "Axis": "dbn.ipynb",
         "axes": "diffdrive.ipynb",
         "plane": "diffdrive.ipynb",
         "ray": "diffdrive.ipynb",
         "show_3d": "diffdrive.ipynb",
         "read_image": "diffdrive.ipynb",
         "conv2": "diffdrive.ipynb",
         "Variables": "discrete.ipynb",
         "DiscreteKey": "discrete.ipynb",
         "show": "display.ipynb",
         "pretty": "display.ipynb",
         "randomImages": "display.ipynb",
         "ROBOTS": "display.ipynb",
         "BG_COLOR": "driving.ipynb",
         "GRID_COLOR": "driving.ipynb",
         "read_ply": "driving.ipynb",
         "sys_byteorder": "driving.ipynb",
         "ply_dtypes": "driving.ipynb",
         "read_lidar_points": "driving.ipynb",
         "cloud_layout": "driving.ipynb",
         "visualize_cloud": "driving.ipynb",
         "gen_color_palette": "driving.ipynb",
         "COLOR_OPTIONS": "driving.ipynb",
         "visualize_clouds": "driving.ipynb",
         "animation_layout": "driving.ipynb",
         "visualize_clouds_animation": "driving.ipynb",
         "td": "html.ipynb",
         "tr": "html.ipynb",
         "th": "html.ipynb",
         "table": "html.ipynb",
         "row": "html.ipynb",
         "ROW": "html.ipynb",
         "vv": "linear.ipynb",
         "gaussian": "logistics.ipynb",
         "map_coords": "logistics.ipynb",
         "show_map": "logistics.ipynb",
         "shelf_width": "logistics.ipynb",
         "shelf_gap": "logistics.ipynb",
         "spacing": "logistics.ipynb",
         "side": "logistics.ipynb",
         "bottom": "logistics.ipynb",
         "top": "logistics.ipynb",
         "base_map": "logistics.ipynb",
         "beacons": "logistics.ipynb",
         "proximity_range": "logistics.ipynb",
         "proximity_map_on": "logistics.ipynb",
         "proximity_map_off": "logistics.ipynb",
         "rfid_range": "logistics.ipynb",
         "rfid_measurement": "logistics.ipynb",
         "out_of_rfid_range": "logistics.ipynb",
         "out_of_bound_map": "logistics.ipynb"}

modules = ["cli.py",
           "dbn.py",
           "diffdrive.py",
           "discrete.py",
           "display.py",
           "driving.py",
           "html.py",
           "linear.py",
           "logistics.py"]

doc_url = "https://gtbook.github.io//gtbook/"

git_url = "https://github.com/gtbook/gtbook/tree/master/"

def custom_doc_links(name): return None
