# AUTOGENERATED! DO NOT EDIT! File to edit: linear.ipynb (unless otherwise specified).

__all__ = ['vv']

# Cell
import gtsam

from typing import List, Tuple, Callable, Dict, Iterable


# Cell
import gtsam
import numpy as np

# Cell
def vv(keys_vectors):
    """Create a VectorValues from a dict"""
    result = gtsam.VectorValues()
    for j, v in keys_vectors.items():
        result.insert(j, v)
    return result